/*
** Copyright (C) 1998,1999 Martin Roesch <roesch@clark.net>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
/* $Id: spo_alert_syslog.c,v 1.2 2000/02/27 03:53:04 roesch Exp $ */

/* spo_alert_syslog 
 * 
 * Purpose:
 *
 * This module sends alerts to the syslog service.
 *
 * Arguments:
 *   
 * Logging mechanism?
 *
 * Effect:
 *
 * Alerts are written to the syslog service with in the facility indicated by
 * the module arguments.
 *
 * Comments:
 *
 * First try
 *
 */

/* your preprocessor header file goes here */
#include "spo_alert_syslog.h"

/* external globals from rules.c */
extern char *file_name;
extern int file_line;

/* If you need to instantiate the preprocessor's data structure, do it here */
SyslogData AlertInfo;

/*
 * Function: SetupSyslog()
 *
 * Purpose: Registers the output plugin keyword and initialization 
 *          function into the output plugin list.  This is the function that
 *          gets called from InitOutputPlugins() in plugbase.c.
 *
 * Arguments: None.
 *
 * Returns: void function
 *
 */
void SetupAlertSyslog()
{
   /* link the preprocessor keyword to the init function in 
      the preproc list */
   RegisterOutputPlugin("alert_syslog", NT_OUTPUT_ALERT, AlertSyslogInit);

#ifdef DEBUG
   printf("Output plugin: Alert-Syslog is setup...\n");
#endif
}


/*
 * Function: AlertSyslogInit(u_char *)
 *
 * Purpose: Calls the argument parsing function, performs final setup on data
 *          structs, links the preproc function into the function list.
 *
 * Arguments: args => ptr to argument string
 *
 * Returns: void function
 *
 */
void AlertSyslogInit(u_char *args)
{
#ifdef DEBUG
   printf("Output: Alert-Syslog Initialized\n");
#endif

   pv.alert_plugin_active = 1;

   /* parse the argument list from the rules file */
   ParseSyslogArgs(args);

#ifdef DEBUG
   printf("Linking syslog alert function to call list...\n");
#endif

   /* Set the preprocessor function into the function list */
   AddFuncToOutputList(AlertSyslog);
}



/*
 * Function: ParseSyslogArgs(char *)
 *
 * Purpose: Process the preprocessor arguements from the rules file and 
 *          initialize the preprocessor's data struct.  This function doesn't
 *          have to exist if it makes sense to parse the args in the init 
 *          function.
 *
 * Arguments: args => argument list
 *
 * Returns: void function
 *
 */
void ParseSyslogArgs(char *args)
{
   char **toks;
   int num_toks;
   int i = 0;

   if(args == NULL)
   {
      fprintf(stderr, "ERROR %s (%d)=> No arguments to alert_syslog preprocessor!\n", file_name, file_line);
      exit(1);
   }

   /* tokenize the argument list */
   toks = mSplit(args, " ", 31, &num_toks, '\\');

   for(i = 0; i < num_toks; i++)
   {
      if(!strncasecmp("LOG_AUTH", toks[i], 8))
      {
#ifdef LOG_AUTH 
         AlertInfo.facility |= LOG_AUTH;
#endif
      }
      else if(!strncasecmp("LOG_CONS", toks[i], 8))
      {
#ifdef LOG_CONS 
         AlertInfo.facility |= LOG_CONS;
#endif
      }
      else if(!strncasecmp("LOG_NDELAY", toks[i], 10))
      {
#ifdef LOG_NDELAY 
         AlertInfo.facility |= LOG_NDELAY;
#endif
      }
      else if(!strncasecmp("LOG_PERROR", toks[i], 10))
      {
#ifdef LOG_PERROR 
         AlertInfo.facility |= LOG_PERROR;
#endif
      }
      else if(!strncasecmp("LOG_PID", toks[i], 7))
      {
#ifdef LOG_PID 
         AlertInfo.facility |= LOG_PID;
#endif
      }
      else if(!strncasecmp("LOG_AUTHPRIV", toks[i], 12))
      {
#ifdef LOG_AUTHPRIV 
         AlertInfo.facility |= LOG_AUTHPRIV;
#endif
      }
      else if(!strncasecmp("LOG_DAEMON", toks[i], 10))
      {
#ifdef LOG_DAEMON 
         AlertInfo.facility |= LOG_DAEMON;
#endif
      }
      else if(!strncasecmp("LOG_LOCAL0", toks[i], 10))
      {
#ifdef LOG_LOCAL0 
         AlertInfo.facility |= LOG_LOCAL0;
#endif
      }
      else if(!strncasecmp("LOG_LOCAL1", toks[i], 10))
      {
#ifdef LOG_LOCAL1 
         AlertInfo.facility |= LOG_LOCAL1;
#endif
      }
      else if(!strncasecmp("LOG_LOCAL2", toks[i], 10))
      {
#ifdef LOG_LOCAL2 
         AlertInfo.facility |= LOG_LOCAL2;
#endif
      }
      else if(!strncasecmp("LOG_LOCAL3", toks[i], 10))
      {
#ifdef LOG_LOCAL3 
         AlertInfo.facility |= LOG_LOCAL3;
#endif
      }
      else if(!strncasecmp("LOG_LOCAL4", toks[i], 10))
      {
#ifdef LOG_LOCAL4 
         AlertInfo.facility |= LOG_LOCAL4;
#endif
      }
      else if(!strncasecmp("LOG_LOCAL5", toks[i], 10))
      {
#ifdef LOG_LOCAL5 
         AlertInfo.facility |= LOG_LOCAL5;
#endif
      }
      else if(!strncasecmp("LOG_LOCAL6", toks[i], 10))
      {
#ifdef LOG_LOCAL6 
         AlertInfo.facility |= LOG_LOCAL6;
#endif
      }
      else if(!strncasecmp("LOG_LOCAL7", toks[i], 10))
      {
#ifdef LOG_LOCAL7 
         AlertInfo.facility |= LOG_LOCAL7;
#endif
      }
      else if(!strncasecmp("LOG_USER", toks[i], 8))
      {
#ifdef LOG_USER 
         AlertInfo.facility |= LOG_USER;
#endif
      }
      else if(!strncasecmp("LOG_EMERG", toks[i], 9))
      {
#ifdef LOG_EMERG 
         AlertInfo.facility |= LOG_EMERG;
#endif
      }
      else if(!strncasecmp("LOG_ALERT", toks[i], 9))
      {
#ifdef LOG_ALERT 
         AlertInfo.facility |= LOG_ALERT;
#endif
      }
      else if(!strncasecmp("LOG_CRIT", toks[i], 8))
      {
#ifdef LOG_CRIT 
         AlertInfo.facility |= LOG_CRIT;
#endif
      }
      else if(!strncasecmp("LOG_ERR", toks[i], 7))
      {
#ifdef LOG_ERR 
         AlertInfo.facility |= LOG_ERR;
#endif
      }
      else if(!strncasecmp("LOG_WARNING", toks[i], 11))
      {
#ifdef LOG_WARNING 
         AlertInfo.facility |= LOG_WARNING;
#endif
      }
      else if(!strncasecmp("LOG_NOTICE", toks[i], 10))
      {
#ifdef LOG_NOTICE 
         AlertInfo.facility |= LOG_NOTICE;
#endif
      }
      else if(!strncasecmp("LOG_INFO", toks[i], 8))
      {
#ifdef LOG_INFO 
         AlertInfo.facility |= LOG_INFO;
#endif
      }
      else if(!strncasecmp("LOG_DEBUG", toks[i], 9))
      {
#ifdef LOG_DEBUG 
         AlertInfo.facility |= LOG_DEBUG;
#endif
      }
      else
      {
         fprintf(stderr, "WARNING %s (%d) => Unrecognized syslog facility: %s\n", file_name, file_line, toks[i]);
      }
   }
}


/*
 * Function: PreprocFunction(Packet *)
 *
 * Purpose: Perform the preprocessor's intended function.  This can be
 *          simple (statistics collection) or complex (IP defragmentation)
 *          as you like.  Try not to destroy the performance of the whole
 *          system by trying to do too much....
 *
 * Arguments: p => pointer to the current packet data struct 
 *
 * Returns: void function
 *
 */
void AlertSyslog(Packet *p, char *msg)
{
   char sip[16];
   char dip[16];

   if(p == NULL)
   {
#ifdef DEBUG
      printf("NULL packet in syslog alert!\n");
#endif
      syslog(AlertInfo.facility, "%s",msg==NULL?"ALERT!":msg);
      return;
   }

   /* have to do this since inet_ntoa is fucked up and writes to
      a static memory location */
   strncpy(sip, inet_ntoa(p->iph->ip_src), 16);
   strncpy(dip, inet_ntoa(p->iph->ip_dst), 16);

   if(p->iph->ip_proto == IPPROTO_ICMP || p->frag_flag)
   {
      if(msg != NULL)
      {
         /* ICMP packets don't get port info... */
         syslog(AlertInfo.facility, "%s: %s -> %s", msg, sip, dip);
      }
      else
      {
         /* ICMP packets don't get port info... */
         syslog(AlertInfo.facility, "ALERT: %s -> %s", sip, dip);
      }
   }
   else
   {
      if(msg != NULL)
      {
         syslog(AlertInfo.facility, "%s: %s:%i -> %s:%i", msg,
                sip, p->sp, dip, p->dp);
      }
      else
      {
         syslog(AlertInfo.facility, "ALERT: %s:%i -> %s:%i",
                sip, p->sp, dip, p->dp);
      }
   }

   return;
}

